<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */

/**
 * DB_DataObject_Extensions_Memcache
 *
 * @author  William Buick <wilbuick@gmail.com>
 * @license http://www.opensource.org/licenses/bsd-license.php New BSD License
 * @link    http://bitbucket.org/wilbuick/db_dataobject_extensions_memcache
 */

/**
 * Require files
 */
require_once 'DB/DataObject.php';
require_once 'DB/DataObject/Extensions/Memcache/Interface.php';
require_once 'DB/DataObject/Extensions/Memcache/Result.php';

/**
 * Class required to implement this extension into your own code.
 *
 * This extension can be implemented in three ways:
 *
 * 1) Use this class to create dataobjects
 *
 *    <code>
 *    $person = DB_DataObject_Extensions_Memcache::factory('person');
 *    $person->memcache_enabled = true; // Is disabled by default
 *    $person->memcache_expire = 120; // Query is cached for two minutes
 *    $person->id = 1;
 *    $person->find();
 *
 *    $person = DB_DataObject_Extensions_Memcache::factory('person');
 *    $person->memcache_enabled = true; // Is disabled by default
 *    $person->id = 1;
 *    $person->find(true); // This query comes from memcache
 *
 *    $person = DB_DataObject_Extensions_Memcache::factory('person');
 *    $person->id = 1;
 *    $person->find(true); // Query is being run on the database (not from memcache)
 *    </code>
 *
 * 2) Implement your own class wrapper for DataObject and have it extend this class.
 *
 *    <code>
 *    class MyDataObject extends DB_DataObject_Extensions_Memcache
 *    {
 *        $this->memcache_enabled = true;
 *        $this->memcache_expire = 120;
 *    }
 *
 *    $person = MyDataObject::get('person', 1); // Query run on database and result stored in memcache
 *    $person = MyDataObject::get('person', 1); // Query result pulled from memcache
 *    </code>
 *
 * 3) If the above does not work, then you will need to create your own DataObject class and have it
 *    implement the DB_DataObject_Extensions_Date_Interface and implement the methods in this class.
 *
 *    <code>
 *    class MyDataObject extends DB_DataObject implements DB_DataObject_Extensions_Memcache_Interface
 *    {
 *        $this->memcache_enabled = true;
 *        $this->memcache_expire = 120;
 *
 *        public function _query($string)
 *        {
 *            return DB_DataObject_Extensions_Memcache::_memcacheQuery(&$this, $string);
 *        }
 *    }
 *
 *    $person = MyDataObject::get('person', 1); // Query run on database and result stored in memcache
 *    $person = MyDataObject::get('person', 1); // Query result pulled from memcache
 *    </code>
 *
 * This extension can be configured as shown above, and also using DB_DataObject standard handling
 * on configuration settings such as a configuration ini file.
 *
 * The configuration options available:
 *
 * 1) memcache_enabled - Boolean value true to enable, false is default.
 * 2) memcache_expiry - Number of seconds to store query result in memcache. 60 seconds is default.
 * 3) memcache_host - Hostname memcache runs on. localhost is default.
 * 4) memcache_port - Port memcache runs on. 11211 is default.
 *
 * @author  William Buick <wilbuick@gmail.com>
 * @license http://www.opensource.org/licenses/bsd-license.php New BSD License
 * @link    http://bitbucket.org/wilbuick/DB_DataObject_Extensions_Date
 */
abstract class DB_DataObject_Extensions_Memcache extends DB_DataObject implements DB_DataObject_Extensions_Memcache_Interface
{

    // DB_DataObject Helpers

    // {{{ _query($string)

    /**
     * Override the DB_DataObject::_query() method with our own _memcacheQuery method so that all
     * queries can be memcached. This is a really bad way to hack this in as the _query() method is
     * private in DB_DataObject, but php4 allows us to do this (all methods being public)
     *
     * @access public
     * @param string $string
     * @return mixed or PEAR_Error
     */
    public function _query($string)
    {
        return self::_memcacheQuery(&$this, $string);
    }

    // }}}

    // Helpers

    // {{{ _memcacheQuery(&$dataobject, $string)

    /**
     * This method can be used to override the usual DB_DataObject::_query() method to cache all or
     * some of the queries run using DB_DataObject in memcache.
     *
     * This method has been designed to be called statically so accepts the DataObject instance as a
     * parameter instead of using $this and extending the DB_DataObject class. This allows both
     * extending DB_DataObject class directly, and also being able to implement this code directly
     * into your project statically
     *
     * @access public
     * @static
     * @throws InvalidArgumentException When $dataobject is not valid
     * @param object $dataobject
     * @param string $string
     * @return mixed or PEAR_Error
     */
    public static function _memcacheQuery(&$dataobject, $string)
    {
        if (!($dataobject instanceof DB_DataObject_Extensions_Memcache_Interface) ||
                !($dataobject instanceof DB_DataObject)) {
            throw new InvalidArgumentException('Unable to run memcache query on this object ' .
                'because it is not an instance of DB_DataObject_Extensions_Memcache_Interface ' .
                'or DB_DataObject');
        }

        global $_DB_DATAOBJECT;

        $enabled = is_bool($dataobject->memcache_enabled) ? $dataobject->memcache_enabled :
            $_DB_DATAOBJECT['CONFIG']['memcache_enabled'];

        if (true == $enabled) {
            if (class_exists('Memcache')) {
                $memcache = new Memcache();

                $memcache_host = isset($dataobject->memcache_host) ? $this->memcache_host :
                    $_DB_DATAOBJECT['CONFIG']['memcache_host'];
                $memcache_port = isset($dataobject->memcache_port) ? $this->memcache_port :
                    $_DB_DATAOBJECT['CONFIG']['memcache_port'];

                if ('' == $memcache_host) {
                    $memcache_host = 'localhost';
                }
                if ('' == $memcache_port) {
                    $memcache_port = '11211';
                }

                if ($memcache->connect($memcache_host, $memcache_port)) {
                    /**
                     * If this is a select query then store the query result into memcached to speed up
                     * subsequent queries.
                     */
                    if (strtolower(substr(trim($string), 0, 6)) === 'select') {
                        $key = sha1($string . 'SOME_SALT');
                        $dataobject->log('Memcached key: ' . $key, PEAR_LOG_INFO);

                        /**
                         * If the result can be retreived from memcache, then read the value back and set the
                         * necessary properties of the dataobject.
                         */
                        if ($result = $memcache->get($key, MEMCACHE_COMPRESSED)) {
                            $dataobject->debug('Query Result Taken From Memcached: ' . $string);
                            //$this->debug('Query Result First Item: ' . print_r($result->data[0], true));

                            $DB =& $_DB_DATAOBJECT['CONNECTIONS'][$dataobject->_database_dsn_md5];

                            $_DB_driver = empty($_DB_DATAOBJECT['CONFIG']['db_driver']) ?
                                'DB' : $_DB_DATAOBJECT['CONFIG']['db_driver'];

                            $_DB_resultid = $GLOBALS['_DB_DATAOBJECT']['RESULTSEQ']++;
                            $_DB_DATAOBJECT['RESULTS'][$_DB_resultid] = $result;
                            $dataobject->_DB_resultid = $_DB_resultid;

                            $dataobject->N = 0;
                            if (method_exists($result, 'numrows')) {
                                if ($_DB_driver == 'DB') {
                                    $DB->expectError(DB_ERROR_UNSUPPORTED);
                                } else {
                                    $DB->expectError(MDB2_ERROR_UNSUPPORTED);
                                }

                                $dataobject->N = $result->numrows();

                                $DB->popExpect();
                            }
                        } else {
                            /**
                             * If the query is not in memcached, then run the query, get the DB_result object
                             * and save it as a DB_Result_Memcache result object. We need to do this because we
                             * cannot store a DB_Result object in memcache because it contains a resource (the
                             * query result) which cannot be stored in memcached because it does not exist
                             * within php. The DB_Result_Memcache object will store all the information
                             * internally within the class and can be safley stored within memcache while
                             * externally still working the same as a DB_Result object.
                             */
                            parent::_query($string);
                            $result = $_DB_DATAOBJECT['RESULTS'][$dataobject->_DB_resultid];
                            $memcached_result = new DB_DataObject_Extensions_Memcache_Result(&$result->dbh, $result->result);
                            $_DB_DATAOBJECT['RESULTS'][$dataobject->_DB_resultid] = $memcached_result;

                            /**
                             * Each dataobject class can define the number of seconds to cache
                             * queries that are run from it. If the dataobject does not define this
                             * value then the default expiry from the global dataobject config is
                             * used.
                             */
                            $expiry = is_int($dataobject->memcache_expire) ? $dataobject->memcache_expire :
                                $_DB_DATAOBJECT['CONFIG']['memcache_expire'];
                            if (!is_int($expiry)) {
                                $expiry = 60;
                            }
                            $memcache->set($key, $memcached_result, MEMCACHE_COMPRESSED, $expiry);

                            $dataobject->debug('Query executed: ' . $string);
                            //$this->debug('Query Result First Item: ' . print_r($memcached_result->data[0], true));
                        }

                        /**
                         * If The query is a select query and run through memcached, then return
                         * here because we do not need to run the rest of this method which only
                         * calls the default DB_DataObject::_query() method.
                         */
                        return;
                    }
                }
            }
        }

        /**
         * If the query is not retrieving any data from the database (not a select query) then
         * it is run normally and not stored in memcache.
         */
        return parent::_query($string);
    }

    // }}}

}

?>
