<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */

/**
 * DB_DataObject_Extensions_Memcache_Result
 *
 * @author  William Buick <wilbuick@gmail.com>
 * @license http://www.opensource.org/licenses/bsd-license.php New BSD License
 * @link    http://bitbucket.org/wilbuick/db_dataobject_extensions_memcache
 */

/**
 * Require files
 */
require_once 'DB.php';

/**
 * This is a custom resultset container for (PEAR) DB that will handle a resultset that is stored in
 * memcache.
 *
 * @author  William Buick <wilbuick@gmail.com>
 * @license http://www.opensource.org/licenses/bsd-license.php New BSD License
 * @link    http://bitbucket.org/wilbuick/DB_DataObject_Extensions_Date
 */
class DB_DataObject_Extensions_Memcache_Result extends DB_Result
{

    // {{{ Properties

    /**
     * Contains the number of columns in the result set.
     *
     * @access private
     * @var int $num_cols
     */
    private $num_cols;

    /**
     * An array of data retrived from the database
     *
     * @access private
     * @var array
     */
    public $data = array();

    // }}}

    // {{{ __construct(&$dbh, $result, $options = array())

    /**
     * This constructor sets the object's properties and reads the database resource into object
     * properties so it can be safley stored in memcached.
     *
     * @param object &$dbh The DB object reference
     * @param resource $result The result resource id
     * @param array $options An associative array with result option
     * @return void
     */
    public function __construct(&$dbh, $result, $options = array())
    {
        $this->dbh =& $dbh;
        $this->fetchmode = $dbh->fetchmode;
        $this->fetchmode_object_class = $dbh->fetchmode_object_class;
        $this->parameters = $dbh->last_parameters;
        $this->query = $dbh->last_query;
        $this->statement = empty($dbh->last_stmt) ? null : $dbh->last_stmt;

        foreach ($options as $key => $value) {
            $this->setOption($key, $value);
        }

        while ($this->dbh->fetchInto($result, $data, DB_FETCHMODE_ASSOC)) {
            $this->data[] = $data;
        }

        //$this->num_cols = count($data);
        $this->num_cols = $this->dbh->numCols($result);

        $err = $this->dbh->freeResult($this->result);
        if (DB::isError($err)) {
            return $err;
        }
    }

    // }}}

    // {{{ fetchRow($fetchmode = DB_FETCHMODE_DEFAULT, $rownum = null)

    /**
     * Fetch a row of data and return it by reference into an array
     *
     * @access public
     * @param int $fetchmode the constant indicating how to format the data
     * @param int $rownum the row number to fetch (index starts at 0)
     * @return mixed an array or object containing the row's data, NULL when the end of the result set is reached
     */
    public function fetchRow($fetchmode = DB_FETCHMODE_DEFAULT, $rownum = null)
    {
        /**
         * If not selecting a specific row and there is an initial limit clause, then start
         * retrieving results from that initial limit_from value
         */
        if ($this->row_counter === null) {
            if (is_null($rownum) && $this->limit_from !== null) {
                $this->row_counter = $this->limit_from;
            } else {
                $this->row_counter = 0;
            }
        }

        /**
         * If limit count is reached, then retireve no more rows even though they may exist.
         */
        if (is_int($this->limit_count)) {
            if ($this->row_counter > $this->limit_count){
                return null;
            }
        }

        /**
         * If the result can be fetched return it, otherwise return null
         */
        if (is_int($rownum)) {
            if (isset($this->data[$rownum])) {
                $arr = $this->data[$rownum];
            } else {
                return null;
            }
        } else if (isset($this->data[$this->row_counter])) {
            $arr = $this->data[$this->row_counter];
            $this->row_counter++;
        } else {
            return null;
        }

        /**
         * Depending on the different fetchmode, return the data in the requested format.
         */
        switch ($fetchmode) {
        case DB_FETCHMODE_ORDERED:
            $numerically_sorted = array();
            foreach ($arr as $item) {
                $numerically_sorted[] = $item;
            }
            return $numerically_sorted;

        case DB_FETCHMODE_OBJECT:
            if (isset($this->fetchmode_object_class)) {
                if ($this->fetchmode_object_class == 'stdClass') {
                    $arr = (object)$arr;
                } else {
                    $arr = new $this->fetchmode_object_class($arr);
                }
                return $arr;
            }

        case DB_FETCHMODE_ASSOC:
        default:
            return $arr;
        }
    }

    // }}}

    // {{{ fetchInto()

    /**
     * Fetch a row of data into an array which is passed by reference
     *
     * @param array &$arr the variable where the data should be placed
     * @param int $fetchmode the constant indicating how to format the data
     * @param int $rownum the row number to fetch (index starts at 0)
     * @return mixed  DB_OK if a row is processed, NULL when the end of the result set is reached
     */
    public function fetchInto(&$arr, $fetchmode = DB_FETCHMODE_DEFAULT, $rownum = null)
    {
        $arr = $this->fetchRow($fetchmode, $rownum);

        if (null === $arr) {
            return null;
        } else {
            return DB_OK;
        }
    }

    // }}}

    // {{{ numCols()

    /**
     * Get the the number of columns in a result set
     *
     * @access public
     * @return int the number of columns.
     */
    public function numCols()
    {
        return $this->num_cols;
    }

    // }}}

    // {{{ numRows()

    /**
     * Get the number of rows in a result set
     *
     * @access public
     * @return int $num_rows the number of rows
     */
    public function numRows()
    {
        $num_rows = count($this->data);

        if (is_int($this->limit_from)) {
            $num_rows = $num_rows - $this->limit_from;
        }
        if (is_int($this->limit_count)) {
            $num_rows = ($num_rows > $this->limit_count) ? $this->limit_count : $num_rows;
        }

        return $num_rows;
    }

    // }}}

    // {{{ nextResult()

    /**
     * Get the next result if a batch of queries was executed
     *
     * @access public
     * @return false
     */
    public function nextResult()
    {
        return false;
    }

    // }}}

    // {{{ free()

    /**
     * Would normally free the database resource, but because a database resource is not stored
     * within this class an exception is thrown instead.
     *
     * @access publc
     * @return boolean
     */
    public function free()
    {
        return true;
    }

    // }}}

    // {{{ tableInfo()

    /**
     * @see DB_common::tableInfo()
     * @deprecated Method deprecated some time before Release 1.2
     * @throws BadMethodCallException Method is deprecated
     */
    public function tableInfo($mode = null)
    {
        throw new BadMethodCallException(sprintf('%s::%s is not implemented because method is deprecated',
            __CLASS__, __METHOD__));
    }

    // }}}

}

?>
